@@ -1,3 +1,4 @@
+
 package Padre::Plugin::Swarm;
 
 use 5.008;
@@ -12,7 +13,8 @@ use Padre::Wx::Icon        ();
 use Padre::Swarm::Geometry ();
 use Padre::Logger;
 
-our $VERSION = '0.093';
+
+our $VERSION = '0.094';
 our @ISA     = 'Padre::Plugin';
 
 use Class::XSAccessor 
@@ -22,9 +24,10 @@ use Class::XSAccessor
 		editor   => 'editor',
 		chat     => 'chat',
 		config   => 'config',
-		message_event => 'message_event',
 		wx => 'wx',
-		transport => 'transport',
+
+		global => 'global',
+		local  => 'local',
 	};
 	
 
@@ -35,16 +38,10 @@ sub connect {
 	# For now - use global, 
 	#  could be Padre::Plugin::Swarm::Transport::Local::Multicast
 	#   based on preferences
-	my $transport = 
-	Padre::Plugin::Swarm::Transport::Global::WxSocket->new(
-		wx => $self->wx,
-		on_recv => sub { $self->on_recv(@_) } ,
-		on_connect => sub { $self->on_transport_connect(@_) },
-		on_disconnect => sub { $self->on_transport_disconnect(@_) }
-	);
-		
-	$self->transport( $transport );
-	$transport->enable;
+	$self->global->enable;
+	$self->local->enable;
+	
+	
 
 	
 }
@@ -52,13 +49,12 @@ sub connect {
 sub disconnect {
 	my $self = shift;
 
-	$self->send( {type=>'leave'} );
-	$self->transport->disable;
-	$self->transport(undef);
+	$self->global->disable;
+	$self->local->disable;
 
 }
 
-sub send {
+sub NOTsend {
 	my $self = shift;
 	my $message = shift;
 	my $mclass = ref $message;
@@ -85,12 +81,14 @@ sub on_transport_connect {
 sub on_transport_disconnect {
 	my ($self) = @_;
 	TRACE( "Swarm transport disconnected" ) if DEBUG;
-	
+	$self->chat->write_unstyled( "swarm transport disconnected!\n" );
+
 }
 
 
 sub on_recv { 
 	my $self = shift;
+	# my $universe = shift;
 	my $message = shift;
 	
 	TRACE( "on_recv handler for " . $message->type ) if DEBUG;
@@ -153,7 +151,7 @@ sub identity {
 # Padre::Plugin Methods
 
 sub padre_interfaces {
-	'Padre::Plugin' => 0.51;
+	'Padre::Plugin' => 0.56;
 }
 
 sub plugin_name {
@@ -191,11 +189,6 @@ sub plugin_large_icon {
 sub menu_plugins_simple {
     my $self = shift;
     return $self->plugin_name => [
-        'Run in Other Editor' => 
-            sub { $self->run_in_other_editor },
-        'Open in Other Editor' => 
-            sub { $self->open_in_other_editor },
-            
         'About' => sub { $self->show_about },
     ];
 }
@@ -215,8 +208,6 @@ SCOPE: {
 		my $wxobj = new Wx::Panel $self->main;
 		$self->wx( $wxobj );
 		$wxobj->Hide;
-		my $message_event  = Wx::NewEventType;
-		$self->message_event($message_event);
 
 		require Padre::Plugin::Swarm::Wx::Chat;
 		require Padre::Plugin::Swarm::Wx::Resources;
@@ -224,56 +215,99 @@ SCOPE: {
 		require Padre::Plugin::Swarm::Wx::Preferences;
 		require Padre::Plugin::Swarm::Transport::Global::WxSocket;
 		require Padre::Plugin::Swarm::Transport::Local::Multicast;
+		require Padre::Plugin::Swarm::Universe;
 		
-		my $config = $self->config_read;
+		my $config = $self->bootstrap_config;
 		$self->config( $config );
 		
+		my $geo = Padre::Swarm::Geometry->new;
+		$self->geometry( $geo );
 		
-		$self->geometry( Padre::Swarm::Geometry->new );
+		my $u_global = Padre::Plugin::Swarm::Universe->new;
+		my $u_local  = Padre::Plugin::Swarm::Universe->new;
+		$self->global($u_global);
+		$self->local($u_local);
 		
-		my $editor = Padre::Plugin::Swarm::Wx::Editor->new();
-		$self->editor($editor);
-		$editor->enable;
-
-		my $chat = Padre::Plugin::Swarm::Wx::Chat->new( $self->main );
-		$self->chat( $chat );
-		$chat->enable;
-
-
-		my $directory = Padre::Plugin::Swarm::Wx::Resources->new(
-			$self->main
+		## Instance the transport but do not connect them - yet
+		my $t_global = 
+		Padre::Plugin::Swarm::Transport::Global::WxSocket->new(
+			token => $self->config->{token},
+			wx => $self->wx,
 		);
-		$self->resources( $directory );
-		$directory->enable;
+		$u_global->transport($t_global);
+	
+		my $t_local = 
+			Padre::Plugin::Swarm::Transport::Local::Multicast->new(
+				token => $self->config->{token},
+				wx    => $self->wx,
+			);
+		$u_local->transport($t_local);
+		
+		
+		$u_global->geometry($geo);
+		$u_local->geometry($geo);
+		
+		## Should this be in global or local?
+		my $editor = Padre::Plugin::Swarm::Wx::Editor->new(
+			transport => $t_global,
+		);
+		$self->editor($editor);
+		$u_global->editor($editor);
 
+		my $g_directory = Padre::Plugin::Swarm::Wx::Resources->new(
+			$self->main,
+			label => 'Global'
+		);
+		$self->resources( $g_directory );
+		$u_global->resources($g_directory);
+		
+		my $g_chat = Padre::Plugin::Swarm::Wx::Chat->new( $self->main,
+				label => 'Global', transport => $self->global->transport
+		 );
+		$u_global->chat($g_chat);
+		
+		my $l_chat = Padre::Plugin::Swarm::Wx::Chat->new( 
+				$self->main,
+				label => 'Local', 
+				transport => $self->local->transport
+		 );
+		$u_local->chat($l_chat);
+		
+		
 		$self->connect();
+
+		
 		1;
 	}
 
 	sub plugin_disable {
 		my $self = shift;
-		$self->chat->disable;
-		$self->chat(undef);
 		
-		$self->resources->disable;
-		$self->resources(undef);
-	
+		eval {
+				$self->global->disable;
+		};
+		if ($@) {
+			TRACE( "Disable global failed $@" ) if DEBUG;
+		}
+		
+		eval { $self->local->disable; };
+		if ($@) {
+			TRACE( "Disable local failed $@" ) if DEBUG;
+		}
+		
 		$self->editor->disable;
 		$self->editor(undef);
 		
 		$self->wx->Destroy;
 		$self->wx(undef);
-		$self->disconnect;
 		
-	
 		undef $instance;
 	
 		
 	}
 }
 
-# TODO Re-anable this when padre can survive plugin_preferences death
-sub NOT_plugin_preferences {
+sub plugin_preferences {
 	my $self = shift;
 	my $wx = shift;
 	if  ( $self->instance ) {
@@ -291,6 +325,30 @@ sub NOT_plugin_preferences {
 	return;
 }
 
+sub bootstrap_config {
+	my $self = shift;
+	my $config = $self->config_read;
+	#warn 'Got ' , join "\t" , %$config;
+	@$config{qw/
+		nickname
+		token
+		transport
+		local_multicast
+		global_server
+		bootstrap
+	/} = (
+		'Anonymous_'.$$,
+		crypt ( rand().$$.time, 'swarm' ) ,
+		'global',
+		'239.255.255.1',
+		'swarm.perlide.org',
+		$VERSION
+		) ;
+		
+	$self->config_write( $config );
+	return $config;
+	
+}
 
 sub editor_enable {
 	my $self = shift;
@@ -303,33 +361,6 @@ sub editor_disable {
 }
 
 
-# oh noes!
-sub run_in_other_editor {
-    my $self = shift;
-    my $ed = $self->current->editor;
-    my $doc = $self->current->document;
-    $self->send(
-        Padre::Swarm::Message->new(
-            type => 'runme',
-            body => $ed->GetText,
-            filename => $doc->filename,
-        )
-    );
-    
-}
-
-sub open_in_other_editor {
-    my $self = shift;
-    my $doc = $self->current->document;
-    my $message = Padre::Swarm::Message->new(
-        type => 'openme',
-        body => $doc->text_get,
-        filename => $doc->filename,
-    );
-    $self->send($message);
-    
-}
-
 sub show_about {
 	my $self = shift;
 
