@@ -4,12 +4,11 @@ use 5.008;
 use strict;
 use warnings;
 use Padre::Wx                        ();
-use Padre::Wx::Directory::SearchCtrl ();
 use Padre::Plugin::Swarm::Wx::Resources::TreeCtrl ();
 use Padre::Logger;
 use Params::Util qw( _INSTANCE ) ;
 
-our $VERSION = '0.093';
+our $VERSION = '0.094';
 our @ISA     = 'Wx::Panel';
 
 use Class::XSAccessor {
@@ -23,6 +22,7 @@ use Class::XSAccessor {
 		previous_dir          => 'previous_dir',
 		project_dir_original  => 'project_dir_original',
 		previous_dir_original => 'previous_dir_original',
+		label => 'label',
 	},
 };
 
@@ -47,13 +47,15 @@ sub plugin { Padre::Plugin::Swarm->instance }
 sub new {
 	my $class = shift;
 	my $main  = shift;
-	
+	my %args = @_;
 	my $self = $class->SUPER::new(
 		$main->directory_panel,
 		-1,
 		Wx::wxDefaultPosition,
 		Wx::wxDefaultSize,
 	);
+	$self->label($args{label});
+	
 	$self->{tree}   = 
 		Padre::Plugin::Swarm::Wx::Resources::TreeCtrl->new($self);
 
@@ -67,7 +69,7 @@ sub new {
 	$self->SetSizerAndFit($sizerh);
 	$sizerh->SetSizeHints($self);
 	$self->Hide;
-	TRACE( "Resource tree Ready - ", $self->tree );
+	TRACE( "Resource tree Ready - ", $self->tree ) if DEBUG;
 	return $self;
 	
 }
@@ -75,33 +77,21 @@ sub new {
 
 sub enable {
 	my $self = shift;
-	TRACE( "Enable" );
+	TRACE( "Enable" ) if DEBUG;
 	my $left = $self->main->directory_panel;
-	my $position = $left->GetPageCount;
-	my $pos = $left->InsertPage( $position, $self, gettext_label(), 0 );
-	my $icon = Padre::Plugin::Swarm->plugin_icon;
 	
-	$left->SetPageBitmap($position, $icon );
-	$left->SetSelection($position);
-
+	$left->show($self);
+	
+	
 	# TODO - use wx event to catch messages. 
 	$self->refresh;
-	Wx::Event::EVT_COMMAND(
-		$self->plugin->wx,
-		-1,
-		$self->plugin->message_event,
-		sub { $self->on_swarm_message(@_) },
-	);
-	
 
-	$self->Show;
-	
 	return $self;
 }
 
 sub disable {
 	my $self = shift;
-	TRACE( "Disabled" );
+	TRACE( "Disabled" ) if DEBUG;
 	my $left = $self->main->directory_panel;
 	my $pos = $left->GetPageIndex($self);
 	$self->Hide;
@@ -125,11 +115,19 @@ sub current {
 }
 
 # Returns the window label
-sub gettext_label {
+sub view_label {
 	my $self = shift;
-	return Wx::gettext('Swarm');
+	return $self->label;
 }
 
+*gettext_label = \&view_label;
+
+sub view_icon {
+	my $icon = Padre::Plugin::Swarm->plugin_icon;
+}
+
+sub view_panel  { 'left' }
+
 # Updates the gui, so each compoment can update itself
 # according to the new state
 sub clear {
@@ -137,15 +135,9 @@ sub clear {
 	return;
 }
 
-sub on_swarm_message {
+sub on_recv {
 	my $self = shift;
-	my $wx = shift;
-	my $event = shift;
-	my $data = $event->GetData;
-	$event->Skip(1);
-	my $message = Storable::thaw( $data );
-	return unless _INSTANCE( $message , 'Padre::Swarm::Message' );
-
+	my $message = shift;
 	my $handler = 'accept_' . $message->type;
 	TRACE( $handler ) if DEBUG;
         if ( $self->can( $handler ) ) {
@@ -158,6 +150,8 @@ sub on_swarm_message {
 	
 }
 
+
+## TODO Perform less revolting redraw when things change
 sub accept_promote {
 	my ($self,$message) = @_;
 	if ( $message->{resource} ) {
@@ -188,7 +182,7 @@ sub accept_leave {
 
 sub refresh {
 	my $self     = shift;
-	TRACE( "Refresh" );	
+	TRACE( "Refresh" ) if DEBUG;
 	my $current  = $self->current;
 	my $document = $current->document;
 
